#!/usr/bin/env python
#
# Browser bookmarks menu as a GNOME panel applet.
#
# (C) 2004 Nigel Tao.
# Licensed under the GNU GPL.

import fileinput
import gconf
import gnome
import gnome.applet
import gnome.ui
import gtk
import gobject
import HTMLParser
import os.path
import re
import sys
import xml.sax


# Encapsulated code to read the Mozilla/Firefox bookmarks file format.
# This class is based on GPL code taken from bookmarks-applet.py
# as found in gnome-python version 1.4.4 (author unknown).
class MozillaFormatBookmarksParser(HTMLParser.HTMLParser):
	def __init__(self, bookmarks_file_name):
		HTMLParser.HTMLParser.__init__(self)
		self.data = ""
		self.root_menu = []
		self.tree_stack = []
		self.item_title = None
		self.item_href = None
		if bookmarks_file_name:
			self.feed(file(bookmarks_file_name, 'r').read())

	def handle_starttag(self, tag, attrs):
		tag = tag.lower()
		if tag == "a":
			self.data = ""
			for tag, value in attrs:
				if tag.lower() == 'href':
					self.item_href = value
		elif tag == "dl":
			new_menu = []
			self.tree_stack.append(new_menu)
			if len(self.tree_stack) > 1:
				self.tree_stack[-2].append((self.item_title, new_menu))
			else:
				self.root_menu = new_menu
		elif tag == "h1" or tag == "h3":
			self.data = ""
		elif tag == "hr":
			self.tree_stack[-1].append((None, None))

	def handle_endtag(self, tag):
		tag = tag.lower()
		if tag == "a":
			self.tree_stack[-1].append((self.data, self.item_href))
		elif tag == "dl":
			del self.tree_stack[-1]
		elif tag == "h1" or tag == "h3":
			self.item_title = self.data

	def handle_data(self, data):
		self.data = self.data + data
# end class MozillaFormatBookmarksParser



# Encapsulated code to read the Mozilla/Firefox bookmarks file format.
# This class is based on GPL code taken from bookmarks-applet.py
# as found in gnome-python version 1.4.4 (author unknown).
class EpiphanyFormatBookmarksParser(xml.sax.ContentHandler):
	def __init__(self, bookmarks_file_name):
		xml.sax.ContentHandler.__init__(self)
		
		self.subjects = {}
		self.bookmarks_with_no_subjects = []
		self.data = ""
		self.title = None
		self.href = None
		
		self.root_menu = []
		parser = xml.sax.make_parser()
		parser.setContentHandler(self)
		if bookmarks_file_name:
			parser.parse(bookmarks_file_name)

	def startElement(self, name, attrs):
		self.data = ""
		if name == "item":
			self.title = None
			self.href = None
			self.no_subject = True

	def endElement(self, name):
		if name == "title":
			self.title = self.data
		elif name == "link":
			if self.href == None:
				self.href = self.data
		elif name == "ephy:smartlink":
			self.href = self.data
		elif name == "dc:subject":
			self.no_subject = False
			s = self.data
			if not self.subjects.has_key(s):
				self.subjects[s] = []
			self.subjects[s].append((self.title, self.href))
		elif name == "item":
			if self.no_subject:
				self.bookmarks_with_no_subjects.append((self.title, self.href))

	def characters(self, data):
		self.data = self.data + data
	
	def endDocument(self):
		self.root_menu = []
		keys = self.subjects.keys()
		keys.sort()
		for k in keys:
			b = self.subjects[k]
			b.sort(sort_by_title)
			self.root_menu.append((k, b))
		self.bookmarks_with_no_subjects.sort(sort_by_title)
		self.root_menu.extend(self.bookmarks_with_no_subjects)

def sort_by_title(x, y):
	return cmp(x[0].lower(), y[0].lower())
# end class EpiphanyFormatBookmarksParser



# begin browser-specific file location methods
def get_firefox_bookmarks_file_name():
	try:
		firefox_dir = os.path.expanduser("~/.mozilla/firefox/")
		path_pattern = re.compile("^Path=(.*)")
		for line in fileinput.input(firefox_dir + "profiles.ini"):
			if line == "":
				break
			match_obj = path_pattern.search(line)
			if match_obj:
				return firefox_dir + match_obj.group(1) + "/bookmarks.html"
	finally:
		fileinput.close()
	return None


def get_mozilla_bookmarks_file_name():
	default_profile_dir = os.path.expanduser("~/.mozilla/default")
	for d in os.listdir(default_profile_dir):
		fn = os.path.join(default_profile_dir, d, "bookmarks.html")
		if os.path.exists(fn):
			return fn
	return None


def get_epiphany_bookmarks_file_name():
	return os.path.expanduser("~/.gnome2/epiphany/bookmarks.rdf")
# end browser-specific file location methods



def get_http_handler():
	return gconf.client_get_default().get_string("/desktop/gnome/url-handlers/http/command")


def get_bookmarks():
	hh = get_http_handler()
	if hh.startswith("firefox"):
		return MozillaFormatBookmarksParser(get_firefox_bookmarks_file_name()).root_menu
	elif hh.startswith("mozilla"):
		return MozillaFormatBookmarksParser(get_mozilla_bookmarks_file_name()).root_menu
	elif hh.startswith("epiphany"):
		return EpiphanyFormatBookmarksParser(get_epiphany_bookmarks_file_name()).root_menu
	else:
		return []


def applet_factory(applet, iid):
	root_menu = gtk.Menu()
	root_menu.connect("hide", on_menu_hidden)
	root_menu.connect("show", on_menu_shown)
	
	root_menu_item = gtk.MenuItem("Bookmarks")
	root_menu_item.set_submenu(root_menu)

	menubar = gtk.MenuBar()
	menubar.connect("button-press-event", on_menubar_click)
	menubar.append(root_menu_item)
	gtk.rc_parse_string('''
		style "browser-bookmarks-menubar-style"
		{
			GtkMenuBar::shadow-type = none
			GtkMenuBar::internal-padding = 0
		}
		class "GtkMenuBar" style "browser-bookmarks-menubar-style"''')
	gobject.type_register(menubar.__class__)
	
	applet.add(menubar)
	applet.show_all()
	applet.connect("destroy", on_applet_destroy, None)
	
	# funky right-click menu
	menuXml = """
	<popup name="button3">
	        <menuitem
	        	name="Browser Bookmarks Menu About Item"
	        	verb="BBM About"
	        	_label="_About"
	        	pixtype="stock"
	        	pixname="gnome-stock-about"/>
	</popup>
	"""
	applet.setup_menu(menuXml, [("BBM About", on_about)], applet)
	
	return gtk.TRUE


def fill_root_menu(menu):
	tooltips = gtk.Tooltips()
	tooltips.enable()
	fill_menu(menu, tooltips, get_bookmarks())


def fill_menu(menu, tooltips, contents):
	if len(contents) == 0:
		menu_item = gtk.MenuItem("No Bookmarks")
		menu_item.set_sensitive(False)
		menu.append(menu_item)
	else:
		for (name, href_or_folder) in contents:
			if name == None:
				menu_item = gtk.SeparatorMenuItem()
			else:
				menu_item = gtk.MenuItem(name)
				if href_or_folder.__class__ == [].__class__:
					folder = href_or_folder
					sub_menu = gtk.Menu()
					menu_item.set_submenu(sub_menu)
					fill_menu(sub_menu, tooltips, folder)
				else:
					href = href_or_folder
					tooltips.set_tip(menu_item, href)
					menu_item.connect("activate", url_show, href)
			menu.append(menu_item)


def url_show(menu_item, href):
	os.system(get_http_handler().replace("%s", href) + "&")


def on_applet_destroy(widget, arg):
	del widget


def on_about(component, verb, applet):
	icon = gtk.Image()
	icon.set_from_file("/usr/share/pixmaps/epiphany-bookmarks.png")

	fullname = "Browser Bookmarks Menu"
	version = "0.2"
	copyright = "Copyright (C) 2004 Nigel Tao"
	description = "A menu for your web bookmarks."
	authors = ["Nigel Tao <nigel.tao@myrealbox.com>","","with thanks to", \
		"Gustavo J. A. M. Carneiro", "Johan Dahlin", "Toshio Kuratomi", "Brett Viren"]

	about = gnome.ui.About(fullname, version, copyright, description, authors, None, None, icon.get_pixbuf())
	about.set_icon(icon.get_pixbuf())
	about.show()


def on_menubar_click(widget, event):
	# allow Middle- and Right-Mouse-Button to go through to the applet window
	if event.button != 1:
		widget.emit_stop_by_name("button-press-event")

	return gtk.FALSE


def on_menu_hidden(menu):
	menu.foreach(lambda x: menu.remove(x))


def on_menu_shown(menu):
	fill_root_menu(menu)
	menu.show_all()


def printBookmarkList(bookmarks, level):
	s = ".   " * level
	for (name, href_or_folder) in bookmarks:
		if name == None:
			print s + "-----"
		else:
			if href_or_folder.__class__ == [].__class__:
				folder = href_or_folder
				print s + "[" + name + "]"
				printBookmarkList(folder, level + 1)
			else:
				href = href_or_folder
				print s + name + "  ->  " + href


if __name__ == "__main__":
	if len(sys.argv) == 2:
		if sys.argv[1] == "-print":
			printBookmarkList(get_bookmarks(), 0)

		elif sys.argv[1] == "-window":
			main_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
			main_window.set_title("Browser Bookmarks Menu")
			main_window.connect("destroy", gtk.main_quit)
			
			applet = gnome.applet.Applet()
			applet_factory(applet, None)
			applet.reparent(main_window)
			
			main_window.show_all()
			gtk.main()

	else:
		gnome.applet.bonobo_factory( \
				"OAFIID:Browser_Bookmarks_Menu_Factory", \
				gnome.applet.Applet.__gtype__, \
				"browser_bookmarks_menu", \
				"0", \
				applet_factory)
